#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "Array.h"
#include "Sort.h"


/* Prototypes */

/* ------------------------------------------------------------------------- *
 * Print the given array on stdout.
 *
 * PARAMETERS
 * array        Array to sort
 * length       Number of elements in the array
 * ------------------------------------------------------------------------- */
static void printArray(int* array, size_t length);

/* ------------------------------------------------------------------------- *
 * Compute the CPU time (in seconds) used by the insertionSort function.
 *
 * PARAMETERS
 * array        Array to sort
 * length       Number of elements in the array
 * ------------------------------------------------------------------------- */
static void timeInsertionSort(int* array, size_t length);



/* Functions */

static void printArray(int* array, size_t length)
{
    printf("[");
    for(size_t i=0; i<length; i++)
        printf("%d, ", array[i]);
    printf("]\n");
}


static void timeInsertionSort(int* array, size_t length)
{
    //Print small array
    if(length <= 10)
    {
        printf("Array to be sorted:\n");
        printArray(array, length);
    }

    //Time the actual sorting
    clock_t start = clock();
    insertionSort(array, length);
    double sec = ((double) (clock() - start)) / CLOCKS_PER_SEC;

    //Print small array
    if(length <= 10)
    {
        printf("Sorted array:\n");
        printArray(array, length);
    }

    //Print sorting time
    printf("Sorting time: %f\n", sec);
}





/* ------------------------------------------------------------------------- *
 * Main
 * ------------------------------------------------------------------------- */
int main(int argc, char** argv)
{


    /* ---------- Command line part -----------*/
    // Parsing command line arguments
    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s <array size>\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    size_t length = (size_t) atoi(argv[1]);

    /* ---------- Main program -----------*/
    srand(time(NULL));//Use an integer seed to get a fix sequence

    printf("Size of the array: %d\n", (int) length);
    int* randomArray = createRandomArray(length);
    int* sortedArray = createNearlySortedArray(length);
    int* fewUniquesArray = createFewUniquesArray(length);

    printf("--Random array:\n");
    timeInsertionSort(randomArray, length);
    printf("--Nearly sorted array:\n");
    timeInsertionSort(sortedArray, length);
    printf("--Few uniques array:\n");
    timeInsertionSort(fewUniquesArray, length);

    free(randomArray);
    free(sortedArray);
    free(fewUniquesArray);

    return 0;
}
